<?php
# api: phrep
# title: NAMED@ parameters
# description: Constructs :named=>parameter function calls.
#
# Permits a $var=>value and Ruby-style :name=>value parameter list
# for function calls. Converts named parameters into array as last
# argument (for variadic funcs).
#
# Use via:
#    NAMED@(func($x, $y, :z=>$z, w=>$w))
#
# Alternative:
#    NAMED@(func, arg, arg, ...)
#
# Becomes:
#    func($x, $y, ["z" => $z, "w" => $w]);
#
# Also permits literal `$arg=123` or `raw=>123` or just `x=5` named params.
#

namespace macro;
$proc->defines["NAMED@"] = [["args..."], "return macro\\NAMED(\$args, \$token);"];

/**
 *
 */
function NAMED($args, $token) {

    // Syntax NAMED@(funcname(...))
    if (count($args) == 1) {
        $name = array_shift($token)[1];
        array_shift($token);  // opening '('
        array_pop($token);    // closing ')'
        $args = \io\Token::args($token);
        $args = array_map('\\io\\Token::join', $args);
    }
    // Already a NAMED(func, arg, arg...) list
    else {
        $name = array_shift($args);
    }
    
    // Assemble param list
    $list = [];
    $named = [];
    foreach ($args as $a) {
        if (preg_match("/^[\$\:]?(\w+)\s*=>?\s*(.+)$/s", $a, $uu)) {
            $named[] = "'$uu[1]' => $uu[2]";
        }
        else {
            $list[] = $a;
        }
    }
    if ($named) { // if not empty
        $list[] = "[" . join(", ", $named) . "]";
    }
    return "$name(" . join(", ", $list) . ")";
}

