<?php
# api: phrep
# type: callback
# title: PARAMETERIZE@() macro
# description: Splits variables out of SQL string, returns SQL=>VAR list
#
# Usage:
#     $db->query(SQL@( SELECT :$v FROM z WHERE x=$x AND y IN (??$y) ));
#
# Becomes:
#     $db->query(' SELECT :? FROM z WHERE x=? AND y IN (??) ', $v, $x, $y);
#
#
# · Now obviously this specific macro variant depends on a query function
#   which understands a list of `'SQL', $val, $val, $val...` arguments.
#
# · Else wrap the macro reference in [ and ] for array syntax.
#
# · The benefit becomes more obvious for queries with lengthy parameter
#   lists, when the parameter order becomes unreadable else.
#
# · And the shorter SQL@ macro name is probably more useful.
#


namespace macro;

// Add macro name
$proc->defines["SQL@"] = [["raw..."], 'return macro\\PARAMETERIZE($token, $raw);'];
$proc->defines["PARAMETERIZE@"] = [["raw..."], 'return macro\\PARAMETERIZE($token, $raw);'];

// Define trivial helper
function PARAMETERIZE($token, $raw, $join=", ") {
    $r = [""];
    $which = [T_VARIABLE, T_CONSTANT_ENCAPSED_STRING, T_DNUMBER, T_LNUMBER];
    foreach (\io\Token::split_expr($token, $which) as $pair) {
        list($sql, $var) = $pair;
        $r[0] .= $sql;
        if ($var) {
            if (substr($r[0], -1, 1) !== "?") {
                $r[0] .= "?";
            }
            $r[] = $var;
        }
    }
    $r[0] = var_export($r[0], TRUE);
    return join($join, $r);
}

