<?php
#ifdef USE_MACRO_SETTERS

   /**
    * Property accessors for MacroProcessor.
    * (Only in composer bundle build, pointless otherwise.)
    *
    */



   /**
    * Define a preproccessor constant.
    *
    * @param  string  $name   Constant
    * @param  string  $value  Literal value
    *
    */
   function addConstant($name, $value=NULL) {
       assert('is_string($name);');
       assert('is_string($value) || is_null($value);');
       $this->defines[$name] = [null, $value];
   }

   /**
    * Add macro. (In string form, not as callable.)
    *
    * @param  string  $name   Macro name
    * @param  array   $args   Parameter name list
    * @param  string  $code   Substitution block
    *
    */
   function addMacro($name, $args, $code) {
       assert('is_string($name);');
       assert('is_array($args);');
       assert('is_string($code) || is_callable($code);');
       $this->defines[$name] = [$args, $code];
   }

   /**
    * Test for constant or macro.
    *
    * @param  string  $name   Constant/Macro name
    *
    */
   function isDefined($name) {
       assert('is_string($name);');
       return isset($this->defines[$name]);
   }

   /**
    * Set pragma option.
    *
    * @param  string  $name   Option name
    * @param  string  $flag   State
    *
    */
   function setPragma($name, $flag) {
       assert('is_string($name) && array_key_exists($name, $this->pragma);');
       assert('is_scalar($flag);');
       $this->pragma[$name] = $flag;
   }

   /**
    * Get pragma flag.
    *
    * @param  string  $name   Option name
    * @return scalar          Current value
    */
   function getPragma($name) {
       assert('is_string($name);');
       return $this->pragma[$name];
   }


   /**
    * Add include search directory.
    *
    * @param  string  $dir    Path/URL
    *
    */
   function addDir($dir) {
       assert('is_string($dir);');
       $this->dirs[] = $dir;
   }

#endif
