<?php
# t: runthrough
# description: operator precedence


// Predef some constants
$me = new io\MacroExpression([
    "X" => [null, "2"],
    "Y" => [null, "-1"],
    "F1" => [null, "3.5"],
    "F2" => [null, "0.0001"],
    "V1" => [null, "1.2.3"],
    "V2" => [null, "1.9.9"],
    "STR" => [null, "TestString"],
], [
    "test" => "run",
]);

// Expressions to check
$probe = [
    "1"                => "1",
    "1+2+3"            => "6",
    "7-5"              => "2",
    "8 + ~5"           => "2",
    "3 ^ 5"            => "6",
    "5 / 2 * 3"        => "7.5",
    "6 / (2 * 3)"      => "1",
    "2+3*(7-2)*('5'*2-1)+X" => 139,
    "X > Y"            => "1",
    "X - Y"            => "3",
    "1<2 && 3>2"       => "1",
    "2 && 0 || 1"      => "1",
    "0 || 0 && 1"      => "0",
    "(2 || 0) && 1"    => "1",
    "defined('X')"     => "1",
    "V2 < 2.0.0"       => "1",
    "V2 >= V1"         => "1",
    "aac =~ 'aa+C'"    => "1",
    "strpos(abc, c)"   => "2",
    "defined(X)"       => "1",
    "defined('X')"     => "1",
    "!defined(NOPE)"   => "1",
    "pragma(test) =~ run" => "1",
    "STR"              => "TestString",
    "UNDEF"            => "UNDEF",   // kept as literal without further expression
    "UNDEF + X"        => "2",
    "UNDEF * X"        => "0",
    "F2 * F1"          => "0.00035",
    "V1 * V2"          => "2.28",
    "X+X==2*X"         => "1",
    "3*X >= 2*X >= X"  => "1",
    "is_int(1*1)"      => "1",   // won't work with just (1), because it's a "1" in the input stack
    "chdir('..')"      => NULL,  // not an allowed function
    // syntax variations
    "defined X"        => 1,
    "file_exists 'Makefile'"  => 1,
    "0 + file_exists NotFN"  => 0,
    // ternary
    "1 + 0 ? 2 * 2 : 3 * 3"  => 4,
];

// stub
set_error_handler(function($i, $msg) { print "\e[34mNULL\e[0m, $msg";});

// Compare results
foreach ($probe as $expr=>$val) {
    print "\e[2m($expr)\e[0m == ";
    //@in io\MacroExpression::expr $tokens, each, is_string
    //@in io\MacroExpression::group $var, each, is_scalar
    //@in io\MacroExpression::group $ops, end, empty
    //@in io\MacroExpression::run $sum, each, is_numeric
    //@in io\MacroExpression::run $opvals, end, empty
    $r = $me->expr($expr);
    if (assert('$r == $val')) {
        print "\e[1;32m$r (OK)\e[0m \n";
    }
    else {
        print "\e[31m$r (F)\e[0m \n";
    }
}

